local function get_ntt_tag(ntt,tag,default)
	local t = ntt.tags
	if t == nil then return default end
	t = t[tag]
	if t == nil then return default end
	return t
end
local function set_ntt_tag(ntt,tag,val)
	local t = ntt.tags or {}
	t[tag] = val
	ntt.tags = t
end
do
	local gFilters = {}
	local gDefaultQuality = "normal"

	local function get_default(tbl,key,default)
		if tbl[key] == nil then tbl[key] = default end
		return tbl[key]
	end

	local function evt_check_inventory(pEvent)
		local pIndex = pEvent.player_index
		if not gFilters[pIndex] then gFilters[pIndex] = {} end

		local plr = game.players[pIndex]
		local chr = plr.character
		if not (plr.physical_controller_type == defines.controllers.character and plr.character and plr.mod_settings["sbCheat-inventory-hack"].value) then return end
		local lst = {}
		for nInv = 1,chr.get_max_inventory_index() do
			local inv = chr.get_inventory(nInv)
			if inv then
				for _,itm in pairs(inv.get_contents()) do
					local i = get_default(get_default(lst,itm.name,{}),itm.quality,0) + itm.count
					lst[itm.name][itm.quality] = i
				end
			end
		end
		for name,quals in pairs(gFilters[pIndex]) do
			for qual,count in pairs(quals) do
				local dif = count - (lst[name] and lst[name][qual] or 0)
				if dif > 0 then
					chr.insert{name=name,count=dif,quality=qual}
				end
			end
		end
	end

	local function evt_filter_inventory(pEvent)
		local pIndex = pEvent.player_index
		local lst = {}

		local plr = game.players[pIndex]
		if not (plr.controller_type == defines.controllers.character and plr.character and plr.mod_settings["sbCheat-inventory-hack"].value) then return end
		for _,logi in pairs(plr.character.get_logistic_point()) do
			if logi.sections then for _,sect in pairs(logi.sections) do
				if sect.filters and sect.active then for _,fltr in pairs(sect.filters) do
					if fltr.value and fltr.value.name and fltr.min and fltr.min > 0 then
						local qual = fltr.value.quality or gDefaultQuality
						local name = fltr.value.name or ""
						if get_default(get_default(lst,name,{}),qual,fltr.min) < fltr.min then
							lst[name][qual] = fltr.min
						end
					end
				end end
			end end
		end
-- 		game.print(#gFilters[pIndex])
		gFilters[pIndex] = lst
		return evt_check_inventory(pEvent)
	end

	local function evt_player_trash_inventory_changed(pEvent)
		local plr = game.players[pEvent.player_index]
		if not (plr.character and plr.mod_settings["sbCheat-trash-hack"].value) then return end
		local inv = plr.get_inventory(defines.inventory.character_trash)
		if inv then
			inv.clear()
		end
	end

	local function evt_player_cursor_stack_changed(pEvent)
		local plr = game.players[pEvent.player_index]
		if not (plr.controller_type == defines.controllers.character and plr.character and plr.mod_settings["sbCheat-ghost-hack"].value) then return end
		local hnd = plr.cursor_stack
		if hnd and hnd.valid_for_read and hnd.prototype then
			hnd.count = hnd.prototype.stack_size
		else
			local ghst = plr.cursor_ghost
			if ghst and ghst.name then
				if ghst.quality then
					hnd.set_stack{name=ghst.name.name,count=ghst.name.stack_size,quality=ghst.quality.name}
-- 					plr.insert{name=ghst.name.name,count=ghst.name.stack_size,quality=ghst.quality.name}
				else
					hnd.set_stack{name=ghst.name.name,count=ghst.name.stack_size}
-- 					plr.insert{name=ghst.name.name,count=ghst.name.stack_size}
				end
			end
		end
	end

	local function evt_entity_logistic_slot_changed(pEvent)
		return evt_filter_inventory(pEvent)
	end

	local function gui_show_ntt(plr,ntt)
		if (not ntt) or ntt.type ~= "linked-container" then return end
		local plr_gui = plr.gui.relative
		local frame = plr_gui.add{type="frame",name="sbCheat_ntt_gui",caption={"sbCheat.gui_frame"},direction="vertical",anchor={gui=defines.relative_gui_type.linked_container_gui,position=defines.relative_gui_position.right}}
		frame.add{type="button",name="sbCheat_btn_rnd",caption={"sbCheat.btn_rnd"},tooltip={"sbCheat.btn_rnd_desc"}}
		frame.add{type="button",name="sbCheat_btn_next",caption={"sbCheat.btn_next"},tooltip={"sbCheat.btn_next_desc"}}
		frame.add{type="label",name="sbCheat_lbl_id",caption={"sbCheat.lbl_id",ntt.link_id}}
		frame.add{type="label",name="sbCheat_lbl_tags",caption={"sbCheat.lbl_tags",serpent.block(ntt.tags)}}
	end
	local function gui_hide_ntt(plr,ntt)
		local plr_gui = plr.gui.relative["sbCheat_ntt_gui"]
		if plr_gui then
			plr_gui.destroy()
		end
	end

	local gui_open = {
		[defines.gui_type.controller] = evt_filter_inventory,
		[defines.gui_type.entity] = function(pEvent)
			return gui_show_ntt(game.players[pEvent.player_index],pEvent.entity)
		end,
	}

	local gui_close = {
		[defines.gui_type.controller] = evt_filter_inventory,
		[defines.gui_type.entity] = function(pEvent)
			return gui_hide_ntt(game.players[pEvent.player_index],pEvent.entity)
		end,
	}

	local function evt_gui_opened(pEvent)
		local fn = gui_open[pEvent.gui_type]
		if fn then return fn(pEvent) end
	end

	local function evt_gui_closed(pEvent)
		local fn = gui_close[pEvent.gui_type]
		if fn then return fn(pEvent) end
	end

	local gui_click = {
		["sbCheat_btn_rnd"] = function(plr,ntt)
			if ntt.type ~= "linked-container" then return end
			ntt.link_id = math.random(0,0xffffffff)
		end,
		["sbCheat_btn_next"] = function(plr,ntt)
			if ntt.type ~= "linked-container" then return end
			local lst = {}
			lst[ntt.link_id] = -1
			for n,surface in pairs(game.surfaces) do
				for i,e in pairs(surface.find_entities_filtered{type=ntt.type,name=ntt.name,force=plr.force}) do
					lst[e.link_id] = (lst[e.link_id] or 0) +1
				end
			end
			local n = 0
			while true do
				n = n+1
				if (lst[n] or 0) <= 0 then
					ntt.link_id = n
					set_ntt_tag(ntt,"test",n)
					game.print(get_ntt_tag(ntt,"test"))
					return
				end
			end
		end,
	}
	local function evt_gui_click(pEvent)
		local fn = gui_click[pEvent.element.name]
		if not fn then return end
		local plr = game.players[pEvent.player_index]
		return fn(plr,plr.opened)
	end

	script.on_event({
		defines.events.on_player_ammo_inventory_changed,
		defines.events.on_player_armor_inventory_changed,
		defines.events.on_player_gun_inventory_changed,
		defines.events.on_player_main_inventory_changed,
	}, evt_check_inventory)

	script.on_event(defines.events.on_gui_click,evt_gui_click)
	script.on_event(defines.events.on_gui_closed,evt_gui_closed)
	script.on_event(defines.events.on_gui_opened,evt_gui_opened)
	script.on_event(defines.events.on_entity_logistic_slot_changed,evt_entity_logistic_slot_changed)
	script.on_event(defines.events.on_player_trash_inventory_changed,evt_player_trash_inventory_changed)
	script.on_event(defines.events.on_player_cursor_stack_changed,evt_player_cursor_stack_changed)

end

do

	local function res_tech(tech)
		if tech then tech.researched = true end
	end

	local OnceOnly = 60
	script.on_event(defines.events.on_tick,function(pEvent)
		if OnceOnly > 0 then
			OnceOnly = OnceOnly - 1
			return
		end

		for fid,force in pairs(game.forces) do
			for rid,recipe in pairs(force.recipes) do
				if recipe.name:sub(1,8) == "sbCheat-" then
					recipe.enabled = true
				end
			end
			for pid,plr in pairs(force.players) do
				if plr.mod_settings["sbCheat-inventory-hack"].value then
					--res_tech(force.technologies["character-logistic-trash-slots-1"])	-- 0.17
					--res_tech(force.technologies["auto-character-logistic-trash-slots"])	-- 0.17
					res_tech(force.technologies["logistic-robotics"])	-- 0.18

					break
				end
			end
		end

		script.on_event(defines.events.on_tick,nil)
	end)

end