local fmt = string.format
local table_copy = table.deepcopy
local substr = string.sub

local item_sounds = require("__base__.prototypes.item_sounds")

local resist_all = {}

do local i = 0
for k,v in pairs(data.raw["damage-type"]) do
	i = i + 1
	resist_all[i] = {
		type = v.name,
		decrease = 100,
		percent = 100
	}
end end

local function rebrand(name,group,subgroup,order,...)
	for num,itm in pairs{...} do
		itm.name = "sbCheat-"..name
		itm.localised_name = {"sbCheat."..name}
		itm.localised_description = {"sbCheat."..name.."-desc"}
		itm.group = group
		itm.subgroup = subgroup
		itm.order = order
		if itm.flags then
			local fn = #itm.flags
			while fn > 0 do
				if itm.flags[fn] == "hidden" then
					table.remove(itm.flags,fn);
				end
				fn = fn -1
			end
		end
	end
end

local function get_recipe(original)
	local ret = {}
	ret["type"] = original["type"]
	ret["icon"] = original["icon"]
	ret["icon_size"] = original["icon_size"]
	ret["icons"] = original["icons"]
	ret["allow_as_intermediate"] = true
	ret["allow_consumption"] = true
	ret["allow_decomposition"] = true
	ret["allow_inserter_overload"] = true
	ret["allow_intermediates"] = true
	ret["allow_pollution"] = true
	ret["allow_productivity"] = true
	ret["allow_quality"] = true
	ret["allow_speed"] = true
	ret["enabled"] = true
	ret["hide_from_player_crafting"] = false
	ret["hide_from_signal_gui"] = false
	ret["hide_from_stats"] = false
	ret["result_is_always_fresh"] = true
	return ret
end



-- ###################
-- ### ITEM GROUPS ###
-- ###################
do

local grp = table_copy(data.raw["item-group"]["fluids"])
grp.name = "sbCheat-group"
grp.order = "z"
local lst = {grp}
grp = table_copy(data.raw["item-subgroup"]["fluid"])
grp.group = "sbCheat-group"

grp.name = "sbCheat-group-equip"
grp.order = "a"
lst[2] = table_copy(grp)

grp.name = "sbCheat-group-supply"
grp.order = "b"
lst[3] = table_copy(grp)

grp.name = "sbCheat-group-misc"
grp.order = "c"
lst[4] = table_copy(grp)

grp.name = "sbCheat-group-hacks"
grp.order = "d"
lst[5] = table_copy(grp)

grp.name = "sbCheat-group-tiles"
grp.order = "e"
lst[6] = table_copy(grp)

data:extend(lst)

end



-- ##################
-- ### SPIDERTRON ###
-- ##################
do
local item = table_copy(data.raw["item-with-entity-data"]["spidertron"])
local spider = table_copy(data.raw["spider-vehicle"]["spidertron"])
local recipe = table_copy(data.raw.recipe["spidertron"])

rebrand("spider","sbCheat-group","sbCheat-group-equip","_c001" , item,spider,recipe)

recipe.results = { {type="item", name=item.name, amount=1} }
item.place_result = spider.name
spider.minable.result = item.name

spider.max_health = 999999
spider.resistances = table_copy(resist_all)

recipe.energy_required = 1
recipe.enabled = true
recipe.ingredients = {}

data:extend{item,spider,recipe}
end



-- #############
-- ### ARMOR ###
-- #############
do
local armor = table_copy(data.raw.armor["power-armor-mk2"])
local armor_recipe = table_copy(data.raw.recipe["power-armor-mk2"])

rebrand("armor","sbCheat-group","sbCheat-group-equip","_c002" , armor,armor_recipe)

armor_recipe.results = { {type="item", name=armor.name, amount=1} }

armor.infinite = true
-- armor.durability = 999999
armor.resistances = table_copy(resist_all)
armor.inventory_size_bonus = 100

armor_recipe.energy_required = 1
armor_recipe.enabled = true
armor_recipe.ingredients = {}

data:extend{armor,armor_recipe}
end



-- #################
-- ### EQUIPMENT ###
-- #################
do for i,v in pairs{
	{"reactor-equip",	"generator-equipment",	"fission-reactor-equipment",	{
		power = "100PW",
	},{}},
	{"battery-equip",	"battery-equipment",	"battery-mk2-equipment",	{},{
		buffer_capacity = "1PJ",
		input_flow_limit = "1PW",
		output_flow_limit = "1PW",
	}},
	{"shield-equip",	"energy-shield-equipment",	"energy-shield-mk2-equipment",	{
		max_shield_value = 1000000,
		energy_per_shield = "1J",
	},{buffer_capacity="1PJ",input_flow_limit="1PW"}},
	{"roboport-equip",	"roboport-equipment",	"personal-roboport-mk2-equipment",	{
		robot_limit = 1000000,
		construction_radius = 1000,
		charging_station_count = 1000000,
		charging_energy = "1PW",
	},{buffer_capacity="1PJ",input_flow_limit="1PW"}},
	{"exoskeleton-equip",	"movement-bonus-equipment",	"exoskeleton-equipment",	{
		energy_consumption = "1W",
		movement_bonus = 3,
	},{buffer_capacity="1MJ",input_flow_limit="1PW"}},
	{"nightvision-equip",	"night-vision-equipment",	"night-vision-equipment",	{
		energy_input = "1W",
	},{buffer_capacity="1MJ",input_flow_limit="1PW"}},
	{"belt-immunity-equip",	"belt-immunity-equipment",	"belt-immunity-equipment",	{
		energy_consumption = "1W",
	},{buffer_capacity="1MJ",input_flow_limit="1PW"}},
} do
	local iName,iType,iGet,iOverwrite,iEnergy = table.unpack(v)
	local recipe = get_recipe(data.raw["recipe"][iGet])
	local item = table_copy(data.raw["item"][iGet])
	local equip = table_copy(data.raw[iType][iGet])

	rebrand(iName,"sbCheat-group","sbCheat-group-equip",fmt('_c%02d',i) , equip,item,recipe)

	equip.take_result = item.name
	item.place_as_equipment_result = equip.name
	recipe.results = { {type="item", name=item.name, amount=1} }

	equip.shape = {type="full",width=1,height=1}
	item.stack_size = 10000

	for k,v in pairs(iOverwrite) do
		equip[k] = v
	end
	for k,v in pairs(iEnergy) do
		equip.energy_source[k] = v
	end
	equip.energy_source.type = "electric"
	equip.energy_source.drain = nil
	equip.energy_source.input_flow_limit = nil
	equip.energy_source.output_flow_limit = nil

	data:extend{equip,item,recipe}
end end



-- ###################
-- ### CONSTRUCTOR ###
-- ###################
do for i,v in pairs{
	{"constructor",	"construction-robot",	{
		max_payload_size = 10,
	}},
	{"transporter",	"logistic-robot",	{
		max_payload_size = 1000,
	}},
} do
	local iName,iType,iOverwrite = table.unpack(v)
	local recipe = get_recipe(data.raw["recipe"][iType])
	local item = table_copy(data.raw["item"][iType])
	local equip = table_copy(data.raw[iType][iType])

	rebrand(iName,"sbCheat-group","sbCheat-group-misc",fmt('_c%02d',i) , equip,item,recipe)

	equip.energy_per_move = nil
	equip.energy_per_tick = nil
	equip.max_energy = nil
	equip.max_health = 100
	equip.max_speed = 1000
	equip.resistances = table_copy(resist_all)
	equip.speed = 1000
	equip.speed_multiplier_when_out_of_energy = 1
	item.stack_size = 10000

	for k,v in pairs(iOverwrite) do
		equip[k] = v
	end

	equip.minable.result = item.name
	item.place_result = equip.name
	recipe.results = { {type="item", name=item.name, amount=item.stack_size} }

	data:extend{equip,item,recipe}
end end



-- #############
-- ### CHEST ###
-- #############
do
local function proto_copy(iType,iName,iOverwrite,iDelete)
	local ret = table_copy(data.raw[iType][iName])
	if iDelete then for k,v in pairs(iDelete) do
		ret[v] = nil
	end end
	for k,v in pairs(iOverwrite) do
		ret[k] = v
	end
	return ret
end
local function item_copy(iName)
	local itm = data.raw["item"][iName]
	local ret = {type="item"}
	for k,v in pairs{"icon","icons","icon_size","pictures","inventory_move_sound","pick_sound","drop_sound"} do
		ret[v] = itm[v]
	end
	return ret
end
for i,v in pairs{
	{"chest",item_copy("infinity-chest"),proto_copy("container","wooden-chest",{
		type = "infinity-container",
		icon = data.raw["infinity-container"]["infinity-chest"].icon,
		picture = data.raw["infinity-container"]["infinity-chest"].picture,
		erase_contents_when_mined = true,
		gui_mode = "all",
		inventory_size = 1000,
		inventory_type = "with_filters_and_bar",
	})},
	{"provider-chest",item_copy("infinity-chest"),proto_copy("container","wooden-chest",{
		type = "infinity-container",
		icon = data.raw["infinity-container"]["infinity-chest"].icon,
		picture = data.raw["infinity-container"]["infinity-chest"].picture,
		erase_contents_when_mined = true,
		gui_mode = "all",
		inventory_size = 1000,
		inventory_type = "with_filters_and_bar",
		logistic_mode = "passive-provider",
	})},
	{"linked-chest",item_copy("linked-chest"),proto_copy("container","wooden-chest",{
		type = "linked-container",
		icon = data.raw["linked-container"]["linked-chest"].icon,
		picture = data.raw["linked-container"]["linked-chest"].picture,
		gui_mode = "all",
		inventory_size = 100,
		inventory_type = "with_filters_and_bar",
	})},
	{"pipe",item_copy("infinity-pipe"),proto_copy("infinity-pipe","infinity-pipe",{
		gui_mode = "all",
	})},
	{"heater",item_copy("heat-interface"),proto_copy("heat-interface","heat-interface",{
		gui_mode = "all",
	})},
	{"power-generator",item_copy("electric-energy-interface"),proto_copy("electric-energy-interface","electric-energy-interface",{
		gui_mode = "all",
	})},
	{"heat-generator",item_copy("heat-interface"),proto_copy("heat-interface","heat-interface",{
    type = "reactor",
		energy_source = {type="void"},
		consumption = "400GW",
		heating_radius = 10,
	},{"gui_mode"})},
} do
	local iName,item,equip = table.unpack(v)
	local recipe = get_recipe{type="recipe"}

	rebrand(iName,"sbCheat-group","sbCheat-group-supply",fmt('_c%02d',i), equip,item,recipe)

	equip.hidden = nil
	equip.max_health = 100
	equip.resistances = table_copy(resist_all)
	item.hidden = nil
	item.stack_size = 10000

	equip.minable.result = item.name
	item.place_result = equip.name
	recipe.results = { {type="item", name=item.name, amount=1} }

	data:extend{equip,item,recipe}
end end



-- #################
-- ### super mod ###
-- #################
do
for i,v in pairs{
	{"efficiency-module",{ consumption=-10,speed=10,pollution=-10,quality=10,productivity=10 }},
	{"speed-module",{ consumption=-10,speed=10,pollution=-10 }},
	{"productivity-module",{ consumption=-10,speed=10,pollution=-10,productivity=10 }},
	{"quality-module",{ consumption=-10,speed=10,pollution=-10,quality=10 }},
} do
	local iName,iEff = table.unpack(v)
	local recipe = get_recipe(data.raw["recipe"][iName])
	local item = table_copy(data.raw["module"][iName])

	rebrand(iName,"sbCheat-group","sbCheat-group-hacks",fmt('_c%02d',i), item,recipe)

	recipe.results = { {type="item", name=item.name, amount=1} }

	item.category = "efficiency"
	item.tier = 0
	item.stack_size = 10000
	item.effect = iEff

	data:extend{item,recipe}
end
end



-- #######################
-- ### overwrite tiles ###
-- #######################
do for i,v in pairs{
	{"water",				"water-shallow",	data.raw["fluid"]["water"]["icon"]},
	{"landfill",		"landfill",	data.raw["item"]["landfill"]["icon"]},
	{"foundation",	"foundation",	data.raw["item"]["foundation"]["icon"]},
	{"oil",					"oil-ocean-shallow",	data.raw["fluid"]["heavy-oil"]["icon"]},
	{"lava",				"lava",	data.raw["fluid"]["lava"]["icon"]},
	{"yumako-soil",	"wetland-yumako",	data.raw["item"]["artificial-yumako-soil"]["icon"]},
	{"overgrowth-yumako-soil",	"overgrowth-yumako-soil",	data.raw["item"]["overgrowth-yumako-soil"]["icon"]},
	{"jellynut-soil",	"wetland-jellynut",	data.raw["item"]["artificial-jellynut-soil"]["icon"]},
	{"overgrowth-jellynut-soil",	"overgrowth-jellynut-soil",	data.raw["item"]["overgrowth-jellynut-soil"]["icon"]},
} do
	local iName,iTile,iIcon = table.unpack(v)
	local recipe = get_recipe{type="recipe"}
	local item = {type="item"}

	rebrand(iName,"sbCheat-group","sbCheat-group-tiles",fmt('_c%02d',i), item,recipe)

	recipe.results = { {type="item", name=item.name, amount=999} }

	item.drop_sound = item_sounds.landfill_inventory_move
	item.icon = iIcon
	item.inventory_move_sound = item_sounds.landfill_inventory_move
	item.pick_sound = item_sounds.landfill_inventory_pickup
	item.place_as_tile = { result=iTile,condition_size=1,condition={layers={}} }
	item.stack_size = 10000

	recipe.results = { {type="item", name=item.name, amount=item.stack_size} }

	data:extend{item,recipe}
end end
